defmodule Nulla.MediaAttachments.MediaAttachment do
  use Ecto.Schema
  import Ecto.Changeset
  alias Nulla.Snowflake
  alias Nulla.Notes.Note

  @primary_key {:id, :integer, autogenerate: false}
  schema "media_attachments" do
    field :type, :string
    field :mediaType, :string
    field :url, :string
    field :name, :string
    field :width, :integer
    field :height, :integer

    belongs_to :note, Note

    timestamps(type: :utc_datetime)
  end

  @doc false
  def changeset(media_attachment, attrs) do
    media_attachment
    |> cast(attrs, [:type, :mediaType, :url, :name, :width, :height, :note_id])
    |> maybe_put_id()
    |> validate_required([:type, :mediaType, :url, :note_id])
  end

  defp maybe_put_id(changeset) do
    id_in_attrs = get_field(changeset, :id)

    if is_nil(id_in_attrs) do
      change(changeset, id: Snowflake.next_id())
    else
      changeset
    end
  end
end
